import os


def file_name_conversion(file_path: str, extension_to_remove=None) -> str:
    """Convert a `/path/to/file.py` to it's usable string name such as `file`.

    :param file_path: the absolute path of a file preferable the __file__ constant.
    :param extension_to_remove: the string that you find irrelevant and needs to be removed
    ie: `/path/to/file_extension.py`, specifying `_extension` will only return `file`
    :return: converted str stripped
    """

    try:
        open(file_path)
    except FileNotFoundError:
        raise FileNotFoundError(f'Error file: {file_path} does not exists.')

    name, extension = os.path.splitext(os.path.basename(file_path))

    if extension_to_remove:
        return str.lower(str(name.split(extension_to_remove)[0]))

    return str.lower(str(name))



import os
from importlib import util
from typing import Any

"""
Extractor Utility
=================

This utility will allow different objects such as class, 
attributes and configuration values,
from py file using a loader ideology 

"""

# The allowed file type.
__file_type__ = '.py'


def get_attribute(file_path: str, attribute_name: str) -> Any:
    """
    Get a module's attribute, by file path and attribute name.

    :param file_path: the absolute path/location of the file.
    :param attribute_name: the attribute you want to extract from the file/module/class
    :return: Any
    :raises: OSError if file does not exists
    :raises: TypeError if file type is not __file_type__
    """

    file, extension = os.path.splitext(file_path)

    if extension == __file_type__:

        try:

            file = open(file_path)
        except OSError:
            raise OSError([file_path])

        file.close()

        spec = util.spec_from_file_location(attribute_name, file_path)
        module = util.module_from_spec(spec)
        spec.loader.exec_module(module)

        return getattr(module, attribute_name)

    else:

        raise TypeError(f'Invalid file type: {extension} from file: {file}')



import setuptools

with open("README.md", "r") as file:
    long_description = file.read()

setuptools.setup(
    name="gwx-core",
    version="0.0.2",
    author="Jerric Calosor",
    author_email="jerric.calosor@groworx.com.au",
    description="Core libraries and modules for flask projects.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/groworxph/gwx-core",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7'
)



from typing import Tuple

"""
Flask Restplus payload wrapper
==============================
Introduces specific structure to serve as abstract to 
every response payload that will be created,
this is tightly coupled with Flask's implementation, 
requiring that the return value MUST be a type Tuple.
"""


def create(message: str, data: dict, headers=None) -> Tuple[dict, int, dict or None]:
    """A response object indicating that create request is successful,
    ideal use for POST type requests.

    :param message: string value
    :param data: the actual response object containing the necessary
    :param headers: custom headers
    :return: conforms with Flask's restplus resource payload type: Tuple
    """
    return __create_response(message, data, 201, headers)


def not_found(message: str, data: dict, headers=None) -> Tuple[dict, int, dict or None]:
    """A response object indicating that the requested resource is not found,
    ideal use for POST type requests.

    :param message: string value
    :param data: the actual response object containing the necessary
    :param headers: custom headers
    :return: conforms with Flask's restplus resource payload type: Tuple
    """
    return __create_response(message, data, 404, headers)


def success(message: str, data: dict, headers=None) -> Tuple[dict, int, dict or None]:
    """A response object indicating that request is successful,
    ideal use for GET, PUT, PATCH type requests.

    :param message: string value
    :param data: the actual response object containing the necessary
    :param headers: custom headers
    :return: conforms with Flask's restplus resource payload type: Tuple
    """

    return __create_response(message, data, 200, headers)


def update_success_no_content(message: str, headers=None) -> Tuple[dict, int, dict or None]:
    """A response object indicating that update request is successful
        but does not contain data, ideal use for PATCH type requests.
        :param message: string value
        :param headers: custom headers
        :return: conforms with Flask's restplus resource payload type: Tuple
        """
    return __create_response(message, None, 203, headers)


def __create_response(
        message: str,
        data: dict or None,
        code: int or None,
        headers: dict or None
) -> Tuple[dict, int, dict or None]:
    """Create the response object Tuple[], that will passed as payload to flask_restplus Resource class.

        :param message: string value
        :param data: the actual response object containing the necessary
        :param code: http status code values see: https://www.restapitutorial.com/httpstatuscodes.html
        :param headers: custom headers
        :return: MUST conform with Flask's restplus resource payload type: Tuple
        """
    data = {"code": 200 if code is None else code, "message": message, "data": data}

    if data is None:
        data.pop("data")

    if headers is not None and type(headers) is not dict:
        raise TypeError(f'Invalid type: {type(headers)}, should be type {dict}')

    return data, code, headers
